/*  twars v1.0 -- a front end to allow tw 1000 to run with the doorware monitor
 *
 *      Parameters, node_number (the rbbs node number).
 *      The program reads necessary data from dorinfo and timeoff and
 *      builds the arguments for twb. It then spawns a synchronous child
 *      process (twb.exe) passing it those parameters.  I replace all
 *      interior balnks with ascii 255's.  After returning from the child
 *      process, this program spawns an overlay process re-invoking MONITOR1
 *      after first writing POINTSx.DOR. It further supports local mode
 *      operation.
 */

 /* COPYRIGHT (c) TIMOTHY M. PEARSON, 1988.
  * See TWARS.DOC for copyright restrictions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <process.h>
#include <local\std.h>

void main(argc, argv)
 int argc;
 char **argv;
 {
	static char copyright[41] = { "Copyright (c) Timothy M. Pearson, 1988." };
	int n;
	int i;
	int exitcode = 0;
	FILE *fp;
	static char mon_in_file[MAXPATH + 2] = { "TIMEOFF1.DOR" };
	static char rbbs_in_file[MAXPATH + 2] = { "DORINFO1.DEF" };
	static char mon_out_file[MAXPATH + 2] = { "POINTS1.DOR" };
	static char gamename[MAXPATH + 1] = { "twq" };
	static char monitorname[MAXNAME + 1] = { "MONITOR1" };
	static char time_buff[9];
	static char node_number[2] = { "1" };
	static char local_flag[3] = { "/L" };
	char ch;
	char *s;
	char *stopstring;
	int ch2;
	static char pgm[MAXPATH + 1] = { "twars" };
	static unsigned long sign_off_secs = 0;
	       unsigned long hours, minutes, seconds;
	static unsigned long user_number = 0;
	static unsigned long points = 0;
	static unsigned long minutes_allowed = 0;
	static unsigned long null_count = 0;
	static unsigned long graphics_used = 0;
	static char monitor_path[MAXPATH + 1] = { "D:\\RBBS-PC\\" };
	static char use_sound[MAXNAME + 1] = { "N" };
	static char p1[32] = { "\0" };
	static char p2[32] = { "\0" };
	static char p3[2] = { "0" };
	static char p6[32] = { "\0" };
	static char com_port[8] = { "COM1" };
	static unsigned long baud_rate = 300;
	static char first_name[31] = { "\0" };
	static char last_name[31] = { "\0" };

	extern void fatal(char *, char *, int);
	extern void getpname(char *, char *);

	if (_osmajor >= 3)
	   getpname(*argv, pgm);

	/* check for node parameter */
	if (argc > 1) {
	   *++argv;
	   printf("argv = %s\n",*argv);
	   strncpy(&mon_in_file[7], *argv, 1);
	   strncpy(node_number, *argv, 1);
	} else { /* we must be in local mode */
	   /* spawn synchronous child process (the game local mode) */
	   if((exitcode = spawnlp(P_WAIT, gamename, gamename, local_flag,
			  NULL)) == -1)
	      fatal(pgm, gamename, 3);
	   /* spawn overlay process (the monitor local mode) */
	    if((exitcode = spawnlp(P_OVERLAY, monitorname, monitorname,
			   NULL)) == -1)
	      fatal(pgm, monitorname, 3);
	}

	/* try to open monitor file in current dir */
	if ((fp = fopen(mon_in_file, "r")) == NULL)
	   fatal(pgm, mon_in_file,1);

	/* read in the fields */
	n = fscanf(fp," %D %D %D %D %D %D %s %s",&sign_off_secs,
						 &user_number,
						 &points,
						 &minutes_allowed,
						 &null_count,
						 &graphics_used,
						 monitor_path,
						 use_sound);
	if (n == EOF || n == 0) {
	   printf("\n Read for monpass failed with n = %d\n",n);
	}

	if (fclose(fp))
	   fatal(pgm,mon_in_file,1);

	/* build rbbs's dorinfox file name */
	if (argc > 1)
	   strncpy(&rbbs_in_file[7],node_number,1);

	/* open rbbs's dorinfx file */
	if ((fp = fopen(rbbs_in_file,"r")) == NULL)
	   fatal(pgm, rbbs_in_file, 1);

	/* read in the fields */
	ch2 = fgetc(fp);
	n = 0;
	while (n < 3) {
	  if ((char)ch2 == '\n')
	     n++;
	  if (n < 3)
	     ch2 = fgetc(fp);
	}  /* skip over first 3 lines in file */
	n = fscanf(fp," %s %D",com_port, &baud_rate);
	if (n == EOF || n == 0)
	   printf("Error reading com port or baud rate from DORINFX. n=%d",n);

	/* skip over rest of com info line and next line */
	n = 0;
	ch2 = fgetc(fp);
	while (n < 2) {
	   if((char)ch2 == '\n')
	      n++;
	   if (n < 2)
	      ch2 = fgetc(fp);
	}

	/* build player first name and handle interior blanks */
	n = i = 0;
	while ((char)(ch2 = fgetc(fp)) != '\n') {
	   if ((char)ch2 != ' ' || n != 0) {
	      first_name[i] = (char)ch2;
	      i++;
	      n = 1;
	   }
	}
	first_name[i] = '\0';
	i--;
	for ( ; first_name[i] == ' '; i--)
	   first_name[i] = '\0';

	while ((s = strchr(first_name,' ')) != NULL)
	   *s = '\255'; /* fix up interior blanks */

	/* build player last name and handle interior blanks */
	n = i = 0;
	while ((char)(ch2 = fgetc(fp)) != '\n') {
	   if ((char)ch2 != ' ' || n != 0) {
	      last_name[i] = (char)ch2;
	      i++;
	      n = 1;
	   }
	}
	last_name[i] = '\0';
	i--;
	for ( ;last_name[i] == ' '; i--)
	   last_name[i] = '\0';

	while ((s = strchr(last_name,' ')) != NULL)
	   *s = '\255'; /* fix up interior blanks */

	if (fclose(fp))
	   fatal(pgm,rbbs_in_file,1);

	/* set up command line argument pointers */
	if (s=getenv("GAMENAME"))
	   strcpy(gamename, s);

	ultoa(baud_rate,p1,10);
	ultoa(minutes_allowed, p2, 10);
	ultoa(graphics_used, p6, 10);

	/* spawn synchronous child process (the game) */
	if((exitcode = spawnlp(P_WAIT, gamename, gamename, p1, p2, p3,
		       first_name, last_name, p6, NULL)) == -1)
	   fatal(pgm, gamename, 3);

	/* ok ebenezer, we're back. make pointsx.dor */
	/* get the time in ascii */
	_strtime(time_buff);
	hours = strtoul(time_buff,&stopstring,10);
	minutes = strtoul(&time_buff[3],&stopstring,10);
	seconds = strtoul(&time_buff[6],&stopstring,10);
	sign_off_secs = (hours*60*60) + (minutes*60) + seconds;
	ultoa(sign_off_secs, time_buff, 10);

	/* build pointsx file name */
	if (argc > 1)
	   strncpy(&mon_out_file[6], node_number, 1);

	/* open rbbs's dorinfx file */
	if ((fp = fopen(mon_out_file,"w")) == NULL)
	   fatal(pgm, mon_out_file, 1);

	/* write the file */
	fprintf(fp," %s \n 0 \n",time_buff);

	/* close the file */
	if (fclose(fp))
	   fatal(pgm, mon_out_file,3);

	/* spawn overlay process (the monitor) */
	 if((exitcode = spawnlp(P_OVERLAY, monitorname, monitorname,
		       node_number, NULL)) == -1)
	   fatal(pgm, monitorname, 3);

	exit(0);
}

